const { GraphQLObjectType, GraphQLString, GraphQLInt, GraphQLList, GraphQLBoolean } = require('graphql');

const convertGeocode = result => {
  const newResult = { };
  (result.address_components || []).forEach(component => {
    component.types.forEach(type => {
      const newType = type.split('_')
        .map((frag, i) => i > 0 ? `${frag[0].toUpperCase()}${frag.substr(1)}` : frag) // eslint-disable-line
        .join('');
      newResult[newType] = component.long_name;
      newResult[`${newType}Short`] = component.short_name;
    });
  });
  newResult.formattedAddress = result.formatted_address;
  if (result.geometry) {
    if (result.geometry.location) {
      newResult.lat = result.geometry.location.lat;
      newResult.lng = result.geometry.location.lng;
    }
    newResult.locationType = result.geometry.location_type;
  }
  newResult.partialMatch = result.partial_match;
  newResult.placeId = result.place_id;
  newResult.types = result.types;
  return newResult;
};

module.exports = config => {
  const maps = require('./index')(config);

  const queries = { };
  const GeocodeType = new GraphQLObjectType({
    name: 'geocode',
    description: 'geocode description',
    fields: {
      streetNumber: { type: GraphQLString },
      route: { type: GraphQLString },
      locality: { type: GraphQLString },
      administrativeAreaLevel1: { type: GraphQLString },
      administrativeAreaLevel2: { type: GraphQLString },
      country: { type: GraphQLString },
      postalCode: { type: GraphQLString },
      formattedAddress: { type: GraphQLString },
      lat: { type: GraphQLInt },
      lng: { type: GraphQLInt },
      locationType: { type: GraphQLString },
      partialMatch: { type: GraphQLBoolean },
      placeId: { type: GraphQLString },
      types: { type: new GraphQLList(GraphQLString) },
    },
  });
  queries.geocode = {
    type: new GraphQLList(GeocodeType),
    description: 'Query GeocodeType',
    args: {
      address: { type: GraphQLString, description: 'Partial or full address' },
      region: { type: GraphQLString, description: 'Region, e.g. "uk"' },
      language: { type: GraphQLString, description: 'Language, e.g. "en"' },
    },
    resolve: (source, args) => maps('geocode', args)
      .then(result => result.json.results.map(convertGeocode)),
  };

  return {
    queries,
    mutations: queries,
  };
};
